# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from instana_client.models.action_instance_request_parameters import ActionInstanceRequestParameters
from typing import Optional, Set
from typing_extensions import Self

class ActionInstanceRequest(BaseModel):
    """
    ActionInstanceRequest
    """ # noqa: E501
    action_id: StrictStr = Field(description="Action identifier of the action to run.", alias="actionId")
    var_async: Optional[StrictStr] = Field(default=None, description="`\"true\"` if the action should be run in asynchronous mode `\"false\"` otherwise. Default is `\"true\"`.", alias="async")
    event_id: Optional[StrictStr] = Field(default=None, description="Event identifier (incident or issue) associated with the policy.", alias="eventId")
    host_id: StrictStr = Field(description="Agent host identifier on which to run the action.", alias="hostId")
    input_parameters: Optional[List[ActionInstanceRequestParameters]] = Field(default=None, description="Action run input parameters.", alias="inputParameters")
    policy_id: Optional[StrictStr] = Field(default=None, description="Policy identifier that associates the action trigger (incident or issue) to the action to run.", alias="policyId")
    timeout: Optional[StrictStr] = Field(default=None, description="Action run time out. Default is `30 seconds`.")
    __properties: ClassVar[List[str]] = ["actionId", "async", "eventId", "hostId", "inputParameters", "policyId", "timeout"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ActionInstanceRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in input_parameters (list)
        _items = []
        if self.input_parameters:
            for _item_input_parameters in self.input_parameters:
                if _item_input_parameters:
                    _items.append(_item_input_parameters.to_dict())
            _dict['inputParameters'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ActionInstanceRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "actionId": obj.get("actionId"),
            "async": obj.get("async"),
            "eventId": obj.get("eventId"),
            "hostId": obj.get("hostId"),
            "inputParameters": [ActionInstanceRequestParameters.from_dict(_item) for _item in obj["inputParameters"]] if obj.get("inputParameters") is not None else None,
            "policyId": obj.get("policyId"),
            "timeout": obj.get("timeout")
        })
        return _obj





# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from instana_client.models.application_alert_rule import ApplicationAlertRule
from instana_client.models.application_node import ApplicationNode
from instana_client.models.application_time_threshold import ApplicationTimeThreshold
from instana_client.models.custom_payload_field import CustomPayloadField
from instana_client.models.rule_with_threshold_application_alert_rule import RuleWithThresholdApplicationAlertRule
from instana_client.models.tag_filter import TagFilter
from instana_client.models.tag_filter_expression_element import TagFilterExpressionElement
from instana_client.models.threshold import Threshold
from typing import Optional, Set
from typing_extensions import Self

class ApplicationAlertConfigWithMetadata(BaseModel):
    """
    ApplicationAlertConfigWithMetadata
    """ # noqa: E501
    alert_channel_ids: Annotated[List[StrictStr], Field(min_length=0, max_length=1024)] = Field(description="List of IDs of alert channels defined in Instana. Can be left empty.", alias="alertChannelIds")
    alert_channels: Optional[Dict[str, List[StrictStr]]] = Field(default=None, description="Set of alert channel IDs associated with the severity.", alias="alertChannels")
    application_id: Optional[Annotated[str, Field(min_length=0, strict=True, max_length=64)]] = Field(default=None, description="ID of the application that this Smart Alert configuration is applied to.", alias="applicationId")
    applications: Dict[str, ApplicationNode] = Field(description="Selection of applications, services, and endpoints that this Smart Alert configuration is associated with. This selection is connected to the defined `tagFilterExpression` by the logical `AND` operator.")
    boundary_scope: StrictStr = Field(description="Determines the source of the application alert configuration. An `INBOUND` scope refers to consumer-made calls. An `ALL` scope refers to both consumer and internally made calls.", alias="boundaryScope")
    created: Optional[Annotated[int, Field(strict=True, ge=1)]] = Field(default=None, description="Unix timestamp representing the creation time of this revision.")
    custom_payload_fields: Annotated[List[CustomPayloadField], Field(min_length=0, max_length=20)] = Field(description="Custom payload fields to send additional information in the alert notifications. Can be left empty.", alias="customPayloadFields")
    description: Annotated[str, Field(min_length=0, strict=True, max_length=65536)] = Field(description="Description of the application alert configuration. Used as a template for the description of alert/event notifications triggered by this Smart Alert configuration.")
    enabled: Optional[StrictBool] = Field(default=None, description="Flag to indicate whether or not the configuration is enabled.")
    evaluation_type: StrictStr = Field(description="Determines whether calls of the aggregated metrics are grouped by the application, the service, or the endpoint. This also determines whether the resulting events are categorized as an issue on the respective entity of that group.", alias="evaluationType")
    grace_period: Optional[StrictInt] = Field(default=None, description="The duration for which an alert remains open after conditions are no longer violated, with the alert auto-closing once the grace period expires.", alias="gracePeriod")
    granularity: StrictInt = Field(description="The evaluation granularity used for detection of violations of the defined threshold. Defines the size of the tumbling window used.")
    id: Annotated[str, Field(min_length=0, strict=True, max_length=64)] = Field(description="ID of this Application App Alert Config. ")
    include_internal: Optional[StrictBool] = Field(default=None, description="Flag to include Internal Calls. These calls are work done inside a service and correspond to intermediate spans in custom tracing.", alias="includeInternal")
    include_synthetic: Optional[StrictBool] = Field(default=None, description="Flag to include Synthetic Calls. These calls have a synthetic endpoint as their destination, such as calls to health-check endpoints. ", alias="includeSynthetic")
    initial_created: Optional[Annotated[int, Field(strict=True, ge=1)]] = Field(default=None, description="Unix timestamp representing the time of the initial revision.", alias="initialCreated")
    name: Annotated[str, Field(min_length=0, strict=True, max_length=256)] = Field(description="Name of the application alert configuration. Used as a template for the title of alert/event notifications triggered by this Smart Alert configuration.")
    read_only: Optional[StrictBool] = Field(default=None, description="Flag to indicate whether or not the configuration is read-only. Read-only access restricts modification of the config.", alias="readOnly")
    rule: Optional[ApplicationAlertRule] = None
    rules: Optional[Annotated[List[RuleWithThresholdApplicationAlertRule], Field(min_length=1, max_length=1)]] = Field(default=None, description="A list of rules where each rule is associated with multiple thresholds and their corresponding severity levels. This enables more complex alert configurations with validations to ensure consistent and logical threshold-severity combinations.")
    severity: Optional[Annotated[int, Field(le=10, strict=True, ge=5)]] = Field(default=None, description="The severity of the alert when triggered, which is either 5 (Warning), or 10 (Critical).")
    tag_filter_expression: TagFilterExpressionElement = Field(alias="tagFilterExpression")
    tag_filters: Optional[List[TagFilter]] = Field(default=None, alias="tagFilters")
    threshold: Optional[Threshold] = None
    time_threshold: ApplicationTimeThreshold = Field(alias="timeThreshold")
    triggering: Optional[StrictBool] = Field(default=None, description="Optional flag to indicate whether an Incident is also triggered or not.")
    __properties: ClassVar[List[str]] = ["alertChannelIds", "alertChannels", "applicationId", "applications", "boundaryScope", "created", "customPayloadFields", "description", "enabled", "evaluationType", "gracePeriod", "granularity", "id", "includeInternal", "includeSynthetic", "initialCreated", "name", "readOnly", "rule", "rules", "severity", "tagFilterExpression", "tagFilters", "threshold", "timeThreshold", "triggering"]

    @field_validator('boundary_scope')
    def boundary_scope_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['ALL', 'INBOUND']):
            raise ValueError("must be one of enum values ('ALL', 'INBOUND')")
        return value

    @field_validator('evaluation_type')
    def evaluation_type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['PER_AP', 'PER_AP_SERVICE', 'PER_AP_ENDPOINT']):
            raise ValueError("must be one of enum values ('PER_AP', 'PER_AP_SERVICE', 'PER_AP_ENDPOINT')")
        return value

    @field_validator('granularity')
    def granularity_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set([60000, 300000, 600000, 900000, 1200000, 1800000]):
            raise ValueError("must be one of enum values (60000, 300000, 600000, 900000, 1200000, 1800000)")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ApplicationAlertConfigWithMetadata from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each value in applications (dict)
        _field_dict = {}
        if self.applications:
            for _key_applications in self.applications:
                if self.applications[_key_applications]:
                    _field_dict[_key_applications] = self.applications[_key_applications].to_dict()
            _dict['applications'] = _field_dict
        # override the default output from pydantic by calling `to_dict()` of each item in custom_payload_fields (list)
        _items = []
        if self.custom_payload_fields:
            for _item_custom_payload_fields in self.custom_payload_fields:
                if _item_custom_payload_fields:
                    _items.append(_item_custom_payload_fields.to_dict())
            _dict['customPayloadFields'] = _items
        # override the default output from pydantic by calling `to_dict()` of rule
        if self.rule:
            _dict['rule'] = self.rule.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in rules (list)
        _items = []
        if self.rules:
            for _item_rules in self.rules:
                if _item_rules:
                    _items.append(_item_rules.to_dict())
            _dict['rules'] = _items
        # override the default output from pydantic by calling `to_dict()` of tag_filter_expression
        if self.tag_filter_expression:
            _dict['tagFilterExpression'] = self.tag_filter_expression.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in tag_filters (list)
        _items = []
        if self.tag_filters:
            for _item_tag_filters in self.tag_filters:
                if _item_tag_filters:
                    _items.append(_item_tag_filters.to_dict())
            _dict['tagFilters'] = _items
        # override the default output from pydantic by calling `to_dict()` of threshold
        if self.threshold:
            _dict['threshold'] = self.threshold.to_dict()
        # override the default output from pydantic by calling `to_dict()` of time_threshold
        if self.time_threshold:
            _dict['timeThreshold'] = self.time_threshold.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ApplicationAlertConfigWithMetadata from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "alertChannelIds": obj.get("alertChannelIds"),
            "alertChannels": obj.get("alertChannels"),
            "applicationId": obj.get("applicationId"),
            "applications": dict(
                (_k, ApplicationNode.from_dict(_v))
                for _k, _v in obj["applications"].items()
            )
            if obj.get("applications") is not None
            else None,
            "boundaryScope": obj.get("boundaryScope"),
            "created": obj.get("created"),
            "customPayloadFields": [CustomPayloadField.from_dict(_item) for _item in obj["customPayloadFields"]] if obj.get("customPayloadFields") is not None else None,
            "description": obj.get("description"),
            "enabled": obj.get("enabled"),
            "evaluationType": obj.get("evaluationType"),
            "gracePeriod": obj.get("gracePeriod"),
            "granularity": obj.get("granularity") if obj.get("granularity") is not None else 600000,
            "id": obj.get("id"),
            "includeInternal": obj.get("includeInternal"),
            "includeSynthetic": obj.get("includeSynthetic"),
            "initialCreated": obj.get("initialCreated"),
            "name": obj.get("name"),
            "readOnly": obj.get("readOnly"),
            "rule": ApplicationAlertRule.from_dict(obj["rule"]) if obj.get("rule") is not None else None,
            "rules": [RuleWithThresholdApplicationAlertRule.from_dict(_item) for _item in obj["rules"]] if obj.get("rules") is not None else None,
            "severity": obj.get("severity"),
            "tagFilterExpression": TagFilterExpressionElement.from_dict(obj["tagFilterExpression"]) if obj.get("tagFilterExpression") is not None else None,
            "tagFilters": [TagFilter.from_dict(_item) for _item in obj["tagFilters"]] if obj.get("tagFilters") is not None else None,
            "threshold": Threshold.from_dict(obj["threshold"]) if obj.get("threshold") is not None else None,
            "timeThreshold": ApplicationTimeThreshold.from_dict(obj["timeThreshold"]) if obj.get("timeThreshold") is not None else None,
            "triggering": obj.get("triggering")
        })
        return _obj





# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.models.available_metrics import AvailableMetrics

class TestAvailableMetrics(unittest.TestCase):
    """AvailableMetrics unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> AvailableMetrics:
        """Test AvailableMetrics
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `AvailableMetrics`
        """
        model = AvailableMetrics()
        if include_optional:
            return AvailableMetrics(
                metrics = [
                    instana_client.models.metric_metadata.MetricMetadata(
                        category = '', 
                        cross_series_aggregations = [
                            'SUM'
                            ], 
                        description = '', 
                        format = 'NUMBER', 
                        id = '', 
                        infra_tag_category = 'ACE', 
                        label = '', 
                        owner_type = '', 
                        tags = [
                            ''
                            ], )
                    ]
            )
        else:
            return AvailableMetrics(
        )
        """

    def testAvailableMetrics(self):
        """Test AvailableMetrics"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.models.test_result_common_properties import TestResultCommonProperties

class TestTestResultCommonProperties(unittest.TestCase):
    """TestResultCommonProperties unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> TestResultCommonProperties:
        """Test TestResultCommonProperties
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `TestResultCommonProperties`
        """
        model = TestResultCommonProperties()
        if include_optional:
            return TestResultCommonProperties(
                client_id = '',
                custom_tags = {
                    'key' : ''
                    },
                dns_query_type = '',
                dns_server_name = '',
                errors = [
                    ''
                    ],
                id = '',
                ism_details = {
                    'key' : ''
                    },
                last_errors = instana_client.models.test_last_error.TestLastError(
                    errors = [
                        ''
                        ], 
                    start_time = 56, ),
                location_display_label = '',
                location_id = '',
                run_type = '',
                ssl_days_remaining = '',
                test_common_properties = instana_client.models.test_common_properties.TestCommonProperties(
                    active = True, 
                    application_id = '', 
                    application_ids = [
                        ''
                        ], 
                    application_label = '', 
                    application_labels = [
                        ''
                        ], 
                    created_at = 56, 
                    frequency = 56, 
                    get_website_labels = [
                        ''
                        ], 
                    id = '', 
                    label = '', 
                    location_display_labels = [
                        ''
                        ], 
                    location_ids = [
                        ''
                        ], 
                    location_labels = [
                        ''
                        ], 
                    location_status_list = [
                        instana_client.models.location_status.LocationStatus(
                            location_display_label = '', 
                            location_id = '', 
                            success_rate = 1.337, 
                            success_runs = 56, 
                            total_test_runs = 56, )
                        ], 
                    mobile_application_ids = [
                        ''
                        ], 
                    mobile_application_labels = [
                        ''
                        ], 
                    modified_at = 56, 
                    service_id = '', 
                    type = '', 
                    website_ids = [
                        ''
                        ], 
                    website_labels = [
                        ''
                        ], ),
                test_id = '',
                test_last_error = instana_client.models.test_last_error.TestLastError(
                    errors = [
                        ''
                        ], 
                    start_time = 56, ),
                test_name = ''
            )
        else:
            return TestResultCommonProperties(
                client_id = '',
                test_id = '',
        )
        """

    def testTestResultCommonProperties(self):
        """Test TestResultCommonProperties"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from instana_client.models.api_restricted_application_filter import ApiRestrictedApplicationFilter
from typing import Optional, Set
from typing_extensions import Self

class ApiTeamScope(BaseModel):
    """
    ApiTeamScope
    """ # noqa: E501
    access_permissions: Optional[List[StrictStr]] = Field(default=None, alias="accessPermissions")
    action_filter: Optional[StrictStr] = Field(default=None, alias="actionFilter")
    applications: Optional[List[StrictStr]] = None
    business_perspectives: Optional[List[StrictStr]] = Field(default=None, alias="businessPerspectives")
    infra_dfq_filter: Optional[StrictStr] = Field(default=None, alias="infraDfqFilter")
    kubernetes_clusters: Optional[List[StrictStr]] = Field(default=None, alias="kubernetesClusters")
    kubernetes_namespaces: Optional[List[StrictStr]] = Field(default=None, alias="kubernetesNamespaces")
    log_filter: Optional[StrictStr] = Field(default=None, alias="logFilter")
    mobile_apps: Optional[List[StrictStr]] = Field(default=None, alias="mobileApps")
    restricted_application_filter: Optional[ApiRestrictedApplicationFilter] = Field(default=None, alias="restrictedApplicationFilter")
    slo_ids: Optional[List[StrictStr]] = Field(default=None, alias="sloIds")
    synthetic_credentials: Optional[List[StrictStr]] = Field(default=None, alias="syntheticCredentials")
    synthetic_tests: Optional[List[StrictStr]] = Field(default=None, alias="syntheticTests")
    tag_ids: Optional[List[StrictStr]] = Field(default=None, alias="tagIds")
    websites: Optional[List[StrictStr]] = None
    __properties: ClassVar[List[str]] = ["accessPermissions", "actionFilter", "applications", "businessPerspectives", "infraDfqFilter", "kubernetesClusters", "kubernetesNamespaces", "logFilter", "mobileApps", "restrictedApplicationFilter", "sloIds", "syntheticCredentials", "syntheticTests", "tagIds", "websites"]

    @field_validator('access_permissions')
    def access_permissions_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        for i in value:
            if i not in set(['LIMITED_APPLICATIONS_SCOPE', 'LIMITED_WEBSITES_SCOPE', 'LIMITED_KUBERNETES_SCOPE', 'LIMITED_MOBILE_APPS_SCOPE', 'LIMITED_INFRASTRUCTURE_SCOPE', 'LIMITED_SYNTHETICS_SCOPE', 'LIMITED_BIZOPS_SCOPE', 'LIMITED_GEN_AI_SCOPE', 'LIMITED_AUTOMATION_SCOPE', 'LIMITED_LOGS_SCOPE', 'LIMITED_ALERT_CHANNELS_SCOPE', 'LIMITED_VSPHERE_SCOPE', 'LIMITED_PHMC_SCOPE', 'LIMITED_POWERVC_SCOPE', 'LIMITED_ZHMC_SCOPE', 'LIMITED_PCF_SCOPE', 'LIMITED_OPENSTACK_SCOPE', 'LIMITED_SAP_SCOPE', 'LIMITED_NUTANIX_SCOPE', 'LIMITED_XENSERVER_SCOPE', 'LIMITED_WINDOWS_HYPERVISOR_SCOPE', 'LIMITED_LINUX_KVM_HYPERVISOR_SCOPE', 'LIMITED_AI_GATEWAY_SCOPE', 'LIMITED_SERVICE_LEVEL_SCOPE']):
                raise ValueError("each list item must be one of ('LIMITED_APPLICATIONS_SCOPE', 'LIMITED_WEBSITES_SCOPE', 'LIMITED_KUBERNETES_SCOPE', 'LIMITED_MOBILE_APPS_SCOPE', 'LIMITED_INFRASTRUCTURE_SCOPE', 'LIMITED_SYNTHETICS_SCOPE', 'LIMITED_BIZOPS_SCOPE', 'LIMITED_GEN_AI_SCOPE', 'LIMITED_AUTOMATION_SCOPE', 'LIMITED_LOGS_SCOPE', 'LIMITED_ALERT_CHANNELS_SCOPE', 'LIMITED_VSPHERE_SCOPE', 'LIMITED_PHMC_SCOPE', 'LIMITED_POWERVC_SCOPE', 'LIMITED_ZHMC_SCOPE', 'LIMITED_PCF_SCOPE', 'LIMITED_OPENSTACK_SCOPE', 'LIMITED_SAP_SCOPE', 'LIMITED_NUTANIX_SCOPE', 'LIMITED_XENSERVER_SCOPE', 'LIMITED_WINDOWS_HYPERVISOR_SCOPE', 'LIMITED_LINUX_KVM_HYPERVISOR_SCOPE', 'LIMITED_AI_GATEWAY_SCOPE', 'LIMITED_SERVICE_LEVEL_SCOPE')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ApiTeamScope from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of restricted_application_filter
        if self.restricted_application_filter:
            _dict['restrictedApplicationFilter'] = self.restricted_application_filter.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ApiTeamScope from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "accessPermissions": obj.get("accessPermissions"),
            "actionFilter": obj.get("actionFilter"),
            "applications": obj.get("applications"),
            "businessPerspectives": obj.get("businessPerspectives"),
            "infraDfqFilter": obj.get("infraDfqFilter"),
            "kubernetesClusters": obj.get("kubernetesClusters"),
            "kubernetesNamespaces": obj.get("kubernetesNamespaces"),
            "logFilter": obj.get("logFilter"),
            "mobileApps": obj.get("mobileApps"),
            "restrictedApplicationFilter": ApiRestrictedApplicationFilter.from_dict(obj["restrictedApplicationFilter"]) if obj.get("restrictedApplicationFilter") is not None else None,
            "sloIds": obj.get("sloIds"),
            "syntheticCredentials": obj.get("syntheticCredentials"),
            "syntheticTests": obj.get("syntheticTests"),
            "tagIds": obj.get("tagIds"),
            "websites": obj.get("websites")
        })
        return _obj





# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.api.application_alert_configuration_api import ApplicationAlertConfigurationApi


class TestApplicationAlertConfigurationApi(unittest.TestCase):
    """ApplicationAlertConfigurationApi unit test stubs"""

    def setUp(self) -> None:
        self.api = ApplicationAlertConfigurationApi()

    def tearDown(self) -> None:
        pass

    def test_create_application_alert_config(self) -> None:
        """Test case for create_application_alert_config

        Create Smart Alert Config
        """
        pass

    def test_delete_application_alert_config(self) -> None:
        """Test case for delete_application_alert_config

        Delete Smart Alert Config
        """
        pass

    def test_disable_application_alert_config(self) -> None:
        """Test case for disable_application_alert_config

        Disable Smart Alert Config
        """
        pass

    def test_enable_application_alert_config(self) -> None:
        """Test case for enable_application_alert_config

        Enable Application Alert Config
        """
        pass

    def test_find_all_active_application_alert_configs(self) -> None:
        """Test case for find_all_active_application_alert_configs

        Get all Smart Alert Configs
        """
        pass

    def test_find_application_alert_config(self) -> None:
        """Test case for find_application_alert_config

        Get Smart Alert Config
        """
        pass

    def test_find_application_alert_config_versions(self) -> None:
        """Test case for find_application_alert_config_versions

        Get Smart Alert Config Versions
        """
        pass

    def test_restore_application_alert_config(self) -> None:
        """Test case for restore_application_alert_config

        Restore Smart Alert Config
        """
        pass

    def test_update_application_alert_config(self) -> None:
        """Test case for update_application_alert_config

        Update Smart Alert Config
        """
        pass

    def test_update_application_historic_baseline(self) -> None:
        """Test case for update_application_historic_baseline

        Recalculate Smart Alert Config Baseline
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from instana_client.models.http_path_segment_matching_rule import HttpPathSegmentMatchingRule
from typing import Optional, Set
from typing_extensions import Self

class HttpEndpointRule(BaseModel):
    """
    Specify custom rule configuration apart from Instana predefined rules. This rule has the highest precedence. This is only available for HTTP endpoints.
    """ # noqa: E501
    enabled: Optional[StrictBool] = Field(default=None, description="Set this flag to `true` if custom rule configurations has to be considered.")
    path_segments: Annotated[List[HttpPathSegmentMatchingRule], Field(min_length=1, max_length=16)] = Field(description="A list of path segment matching rules, each defining how a segment of the HTTP path should be matched. Each object in this array represents a segment rule, allowing for fixed segments, dynamic parameters, wildcards, or unsupported segments.   **UNSUPPORTED**: A path segment that is not recognized by the system.   **FIXED**: This type represents a static, unchanging part of the URL path. For example, `/api/{version}/users`, `api` and `users` would be `FIXED` segment.   **PARAMETER**: This type represents a variable part of the URL path, often used to capture specific parameters or IDs that change with each request. For example, `/api/{version}/users`, `version` would be `PARAMETER` segment. `version` can be `v1`, `v2`, `v3` etc.   **MATCH_ALL**: This type represents a wildcard, capturing all remaining segments from this point onward in the URL path. For example, `/api/{version}/users/*` — Matches all paths like `/api/v1/users/123`. `/api/v3/users/456` etc.   ", alias="pathSegments")
    test_cases: Optional[Annotated[List[StrictStr], Field(min_length=0, max_length=32)]] = Field(default=None, description="To validate whether the the defined custom endpoint rule configuration is working as expected. For example, given a query `/api/*/{version}`, the following test case `/api/anyName/123` will match, while `/otherApi/anyName/123` will not. ", alias="testCases")
    __properties: ClassVar[List[str]] = ["enabled", "pathSegments", "testCases"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of HttpEndpointRule from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in path_segments (list)
        _items = []
        if self.path_segments:
            for _item_path_segments in self.path_segments:
                if _item_path_segments:
                    _items.append(_item_path_segments.to_dict())
            _dict['pathSegments'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of HttpEndpointRule from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "enabled": obj.get("enabled"),
            "pathSegments": [HttpPathSegmentMatchingRule.from_dict(_item) for _item in obj["pathSegments"]] if obj.get("pathSegments") is not None else None,
            "testCases": obj.get("testCases")
        })
        return _obj





# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from instana_client.models.api_tag import ApiTag
from instana_client.models.synthetic_type_configuration import SyntheticTypeConfiguration
from typing import Optional, Set
from typing_extensions import Self

class SyntheticTest(BaseModel):
    """
    SyntheticTest
    """ # noqa: E501
    active: StrictBool
    application_id: Optional[Annotated[str, Field(min_length=0, strict=True, max_length=512)]] = Field(default=None, alias="applicationId")
    application_label: Optional[Annotated[str, Field(min_length=0, strict=True, max_length=512)]] = Field(default=None, alias="applicationLabel")
    application_labels: Optional[List[StrictStr]] = Field(default=None, alias="applicationLabels")
    applications: Optional[List[StrictStr]] = None
    configuration: SyntheticTypeConfiguration
    created_at: Optional[StrictInt] = Field(default=None, alias="createdAt")
    created_by: Optional[Annotated[str, Field(min_length=0, strict=True, max_length=128)]] = Field(default=None, alias="createdBy")
    custom_properties: Optional[Dict[str, StrictStr]] = Field(default=None, alias="customProperties")
    description: Optional[Annotated[str, Field(min_length=0, strict=True, max_length=512)]] = None
    id: Optional[Annotated[str, Field(min_length=1, strict=True, max_length=128)]] = None
    label: Annotated[str, Field(min_length=1, strict=True, max_length=128)]
    location_display_labels: Optional[List[StrictStr]] = Field(default=None, alias="locationDisplayLabels")
    location_labels: Optional[List[StrictStr]] = Field(default=None, alias="locationLabels")
    locations: List[StrictStr]
    mobile_app_labels: Optional[List[StrictStr]] = Field(default=None, alias="mobileAppLabels")
    mobile_apps: Optional[List[StrictStr]] = Field(default=None, alias="mobileApps")
    modified_at: Optional[StrictInt] = Field(default=None, alias="modifiedAt")
    modified_by: Optional[Annotated[str, Field(min_length=0, strict=True, max_length=128)]] = Field(default=None, alias="modifiedBy")
    playback_mode: Optional[StrictStr] = Field(default=None, alias="playbackMode")
    rbac_tags: Optional[List[ApiTag]] = Field(default=None, alias="rbacTags")
    tenant_id: Optional[StrictStr] = Field(default=None, alias="tenantId")
    test_frequency: Annotated[int, Field(le=1440, strict=True, ge=1)] = Field(alias="testFrequency")
    website_labels: Optional[List[StrictStr]] = Field(default=None, alias="websiteLabels")
    websites: Optional[List[StrictStr]] = None
    __properties: ClassVar[List[str]] = ["active", "applicationId", "applicationLabel", "applicationLabels", "applications", "configuration", "createdAt", "createdBy", "customProperties", "description", "id", "label", "locationDisplayLabels", "locationLabels", "locations", "mobileAppLabels", "mobileApps", "modifiedAt", "modifiedBy", "playbackMode", "rbacTags", "tenantId", "testFrequency", "websiteLabels", "websites"]

    @field_validator('playback_mode')
    def playback_mode_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Simultaneous', 'Staggered']):
            raise ValueError("must be one of enum values ('Simultaneous', 'Staggered')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SyntheticTest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "application_label",
            "application_labels",
            "created_at",
            "location_display_labels",
            "location_labels",
            "mobile_app_labels",
            "website_labels",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of configuration
        if self.configuration:
            _dict['configuration'] = self.configuration.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in rbac_tags (list)
        _items = []
        if self.rbac_tags:
            for _item_rbac_tags in self.rbac_tags:
                if _item_rbac_tags:
                    _items.append(_item_rbac_tags.to_dict())
            _dict['rbacTags'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SyntheticTest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "active": obj.get("active"),
            "applicationId": obj.get("applicationId"),
            "applicationLabel": obj.get("applicationLabel"),
            "applicationLabels": obj.get("applicationLabels"),
            "applications": obj.get("applications"),
            "configuration": SyntheticTypeConfiguration.from_dict(obj["configuration"]) if obj.get("configuration") is not None else None,
            "createdAt": obj.get("createdAt"),
            "createdBy": obj.get("createdBy"),
            "customProperties": obj.get("customProperties"),
            "description": obj.get("description"),
            "id": obj.get("id"),
            "label": obj.get("label"),
            "locationDisplayLabels": obj.get("locationDisplayLabels"),
            "locationLabels": obj.get("locationLabels"),
            "locations": obj.get("locations"),
            "mobileAppLabels": obj.get("mobileAppLabels"),
            "mobileApps": obj.get("mobileApps"),
            "modifiedAt": obj.get("modifiedAt"),
            "modifiedBy": obj.get("modifiedBy"),
            "playbackMode": obj.get("playbackMode"),
            "rbacTags": [ApiTag.from_dict(_item) for _item in obj["rbacTags"]] if obj.get("rbacTags") is not None else None,
            "tenantId": obj.get("tenantId"),
            "testFrequency": obj.get("testFrequency"),
            "websiteLabels": obj.get("websiteLabels"),
            "websites": obj.get("websites")
        })
        return _obj





# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.models.infrastructure_entities_result import InfrastructureEntitiesResult

class TestInfrastructureEntitiesResult(unittest.TestCase):
    """InfrastructureEntitiesResult unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> InfrastructureEntitiesResult:
        """Test InfrastructureEntitiesResult
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `InfrastructureEntitiesResult`
        """
        model = InfrastructureEntitiesResult()
        if include_optional:
            return InfrastructureEntitiesResult(
                adjusted_timeframe = instana_client.models.adjusted_timeframe.AdjustedTimeframe(
                    to = 56, 
                    window_size = 0, ),
                can_load_more = True,
                items = [
                    instana_client.models.infrastructure_item.InfrastructureItem(
                        entity_health_info = instana_client.models.entity_health_info.EntityHealthInfo(
                            max_severity = 0, 
                            open_issues = [
                                instana_client.models.event.Event(
                                    end = 1, 
                                    endpoint_service_id = '', 
                                    entity_id = '', 
                                    entity_type = 'Entity10', 
                                    event_configuration_type = '', 
                                    id = '', 
                                    long_form_plugin_id = '', 
                                    metadata = {
                                        'key' : None
                                        }, 
                                    metric_access_id = '', 
                                    plugin = '', 
                                    problem = instana_client.models.problem.Problem(
                                        fix_suggestion = '', 
                                        id = '', 
                                        problem_text = '', 
                                        severity = 56, ), 
                                    rca = {
                                        'key' : None
                                        }, 
                                    start = 1, 
                                    state = '', 
                                    type = '', )
                                ], ), 
                        label = '', 
                        metrics = {
                            'key' : [
                                [
                                    1.337
                                    ]
                                ]
                            }, 
                        plugin = '', 
                        snapshot_id = '', 
                        tags = {
                            'key' : ''
                            }, 
                        time = 56, )
                    ],
                next = instana_client.models.cursor.cursor(),
                total_hits = 0
            )
        else:
            return InfrastructureEntitiesResult(
                items = [
                    instana_client.models.infrastructure_item.InfrastructureItem(
                        entity_health_info = instana_client.models.entity_health_info.EntityHealthInfo(
                            max_severity = 0, 
                            open_issues = [
                                instana_client.models.event.Event(
                                    end = 1, 
                                    endpoint_service_id = '', 
                                    entity_id = '', 
                                    entity_type = 'Entity10', 
                                    event_configuration_type = '', 
                                    id = '', 
                                    long_form_plugin_id = '', 
                                    metadata = {
                                        'key' : None
                                        }, 
                                    metric_access_id = '', 
                                    plugin = '', 
                                    problem = instana_client.models.problem.Problem(
                                        fix_suggestion = '', 
                                        id = '', 
                                        problem_text = '', 
                                        severity = 56, ), 
                                    rca = {
                                        'key' : None
                                        }, 
                                    start = 1, 
                                    state = '', 
                                    type = '', )
                                ], ), 
                        label = '', 
                        metrics = {
                            'key' : [
                                [
                                    1.337
                                    ]
                                ]
                            }, 
                        plugin = '', 
                        snapshot_id = '', 
                        tags = {
                            'key' : ''
                            }, 
                        time = 56, )
                    ],
        )
        """

    def testInfrastructureEntitiesResult(self):
        """Test InfrastructureEntitiesResult"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from instana_client.models.database_integration import DatabaseIntegration
from instana_client.models.stack_trace_item import StackTraceItem
from typing import Optional, Set
from typing_extensions import Self

class SpanExcerpt(BaseModel):
    """
    Information about the logs attached to the call, if available.
    """ # noqa: E501
    data: Dict[str, Dict[str, Any]] = Field(description="Some information about the span, like service name, if it is an http call, then some information about it like, path, method, host, errors etc.")
    database_integrations: Optional[List[DatabaseIntegration]] = Field(default=None, alias="databaseIntegrations")
    duration: Optional[Annotated[int, Field(strict=True, ge=0)]] = Field(default=None, description="The total time taken for the entire operation of a call, from the moment the request was initiated to when the response was received. The time measured is in milliseconds. This is also known as latency of a call.")
    error_count: Optional[Annotated[int, Field(strict=True, ge=0)]] = Field(default=None, description="Represents whether the span is erroneous or not. 0 is not erroneous and 1 is erroneous.", alias="errorCount")
    foreign_parent_id: Optional[StrictStr] = Field(default=None, alias="foreignParentId")
    id: StrictStr = Field(description="The call ID. A unique identifier for an individual call. For example: `1bcad5c82338deaf`.")
    kind: StrictStr = Field(description="There are 4 types of span kind: 1. `ENTRY`: An entry span represents an incoming request into a traced service. 2. `EXIT`: An exit span represents an outgoing request that a service makes to some other service. 3. `INTERMEDIATE`: An intermediate span represents anything that happens inside a traced service where the flow of control neither enters nor leaves that service, but stays inside it. 4. `UNKNOWN`: Instana can't determine the span kind. ")
    name: StrictStr = Field(description="The technical type of the span. For example, `node.http.client` or `jdbc`.")
    parent_id: Optional[StrictStr] = Field(default=None, description="The parent call id, referring to another call in the same trace which triggered the processing associated with this call.", alias="parentId")
    stack_trace: List[StackTraceItem] = Field(description="For an erroneous call, if stack trace is available it will show a list of items containing file, method and line number of the code.", alias="stackTrace")
    start: Optional[Annotated[int, Field(strict=True, ge=1)]] = Field(default=None, description="The timestamp when the call or request was initiated. For example, Unix epoch time in milliseconds `1735532879870` is `Monday, 30 December 2024 04:27:59.870 GMT`")
    __properties: ClassVar[List[str]] = ["data", "databaseIntegrations", "duration", "errorCount", "foreignParentId", "id", "kind", "name", "parentId", "stackTrace", "start"]

    @field_validator('kind')
    def kind_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['UNKNOWN', 'ENTRY', 'EXIT', 'INTERMEDIATE']):
            raise ValueError("must be one of enum values ('UNKNOWN', 'ENTRY', 'EXIT', 'INTERMEDIATE')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SpanExcerpt from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in database_integrations (list)
        _items = []
        if self.database_integrations:
            for _item_database_integrations in self.database_integrations:
                if _item_database_integrations:
                    _items.append(_item_database_integrations.to_dict())
            _dict['databaseIntegrations'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in stack_trace (list)
        _items = []
        if self.stack_trace:
            for _item_stack_trace in self.stack_trace:
                if _item_stack_trace:
                    _items.append(_item_stack_trace.to_dict())
            _dict['stackTrace'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SpanExcerpt from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "data": obj.get("data"),
            "databaseIntegrations": [DatabaseIntegration.from_dict(_item) for _item in obj["databaseIntegrations"]] if obj.get("databaseIntegrations") is not None else None,
            "duration": obj.get("duration"),
            "errorCount": obj.get("errorCount"),
            "foreignParentId": obj.get("foreignParentId"),
            "id": obj.get("id"),
            "kind": obj.get("kind"),
            "name": obj.get("name"),
            "parentId": obj.get("parentId"),
            "stackTrace": [StackTraceItem.from_dict(_item) for _item in obj["stackTrace"]] if obj.get("stackTrace") is not None else None,
            "start": obj.get("start")
        })
        return _obj





# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.models.synthetic_call_with_defaults_config import SyntheticCallWithDefaultsConfig

class TestSyntheticCallWithDefaultsConfig(unittest.TestCase):
    """SyntheticCallWithDefaultsConfig unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> SyntheticCallWithDefaultsConfig:
        """Test SyntheticCallWithDefaultsConfig
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `SyntheticCallWithDefaultsConfig`
        """
        model = SyntheticCallWithDefaultsConfig()
        if include_optional:
            return SyntheticCallWithDefaultsConfig(
                custom_rules = [
                    instana_client.models.synthetic_call_rule.SyntheticCallRule(
                        description = '', 
                        enabled = True, 
                        match_specification = instana_client.models.match_expression_dto.MatchExpressionDTO(
                            type = '', ), 
                        name = '0', )
                    ],
                default_rules = [
                    instana_client.models.synthetic_call_rule.SyntheticCallRule(
                        description = '', 
                        enabled = True, 
                        match_specification = instana_client.models.match_expression_dto.MatchExpressionDTO(
                            type = '', ), 
                        name = '0', )
                    ],
                default_rules_enabled = True
            )
        else:
            return SyntheticCallWithDefaultsConfig(
                custom_rules = [
                    instana_client.models.synthetic_call_rule.SyntheticCallRule(
                        description = '', 
                        enabled = True, 
                        match_specification = instana_client.models.match_expression_dto.MatchExpressionDTO(
                            type = '', ), 
                        name = '0', )
                    ],
                default_rules = [
                    instana_client.models.synthetic_call_rule.SyntheticCallRule(
                        description = '', 
                        enabled = True, 
                        match_specification = instana_client.models.match_expression_dto.MatchExpressionDTO(
                            type = '', ), 
                        name = '0', )
                    ],
        )
        """

    def testSyntheticCallWithDefaultsConfig(self):
        """Test SyntheticCallWithDefaultsConfig"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictBool, StrictInt, StrictStr
from typing import List, Optional
from typing_extensions import Annotated
from instana_client.models.backend_trace_reference import BackendTraceReference
from instana_client.models.call_groups_result import CallGroupsResult
from instana_client.models.get_call_groups import GetCallGroups
from instana_client.models.get_trace_groups import GetTraceGroups
from instana_client.models.get_traces import GetTraces
from instana_client.models.trace_activity_tree_node_details import TraceActivityTreeNodeDetails
from instana_client.models.trace_download_result import TraceDownloadResult
from instana_client.models.trace_groups_result import TraceGroupsResult
from instana_client.models.trace_result import TraceResult

from instana_client.api_client import ApiClient, RequestSerialized
from instana_client.api_response import ApiResponse
from instana_client.rest import RESTResponseType


class ApplicationAnalyzeApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def get_call_details(
        self,
        trace_id: StrictStr,
        call_id: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> TraceActivityTreeNodeDetails:
        """Get call detail

        Use this API endpoint to retrieve a vast information about a call present in a trace.  For more information on Application Analyze please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-analyze.

        :param trace_id: (required)
        :type trace_id: str
        :param call_id: (required)
        :type call_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_call_details_serialize(
            trace_id=trace_id,
            call_id=call_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TraceActivityTreeNodeDetails",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_call_details_with_http_info(
        self,
        trace_id: StrictStr,
        call_id: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[TraceActivityTreeNodeDetails]:
        """Get call detail

        Use this API endpoint to retrieve a vast information about a call present in a trace.  For more information on Application Analyze please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-analyze.

        :param trace_id: (required)
        :type trace_id: str
        :param call_id: (required)
        :type call_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_call_details_serialize(
            trace_id=trace_id,
            call_id=call_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TraceActivityTreeNodeDetails",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_call_details_without_preload_content(
        self,
        trace_id: StrictStr,
        call_id: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get call detail

        Use this API endpoint to retrieve a vast information about a call present in a trace.  For more information on Application Analyze please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-analyze.

        :param trace_id: (required)
        :type trace_id: str
        :param call_id: (required)
        :type call_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_call_details_serialize(
            trace_id=trace_id,
            call_id=call_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TraceActivityTreeNodeDetails",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_call_details_serialize(
        self,
        trace_id,
        call_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if trace_id is not None:
            _path_params['traceId'] = trace_id
        if call_id is not None:
            _path_params['callId'] = call_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/application-monitoring/v2/analyze/traces/{traceId}/calls/{callId}/details',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_call_group(
        self,
        fill_time_series: Annotated[Optional[StrictBool], Field(description="If enabled, fill the missing data points in the metric result with timestamp and value 0.")] = None,
        get_call_groups: Optional[GetCallGroups] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> CallGroupsResult:
        """Get grouped call metrics

         For more information on Application Analyze please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-analyze.

        :param fill_time_series: If enabled, fill the missing data points in the metric result with timestamp and value 0.
        :type fill_time_series: bool
        :param get_call_groups:
        :type get_call_groups: GetCallGroups
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_call_group_serialize(
            fill_time_series=fill_time_series,
            get_call_groups=get_call_groups,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CallGroupsResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_call_group_with_http_info(
        self,
        fill_time_series: Annotated[Optional[StrictBool], Field(description="If enabled, fill the missing data points in the metric result with timestamp and value 0.")] = None,
        get_call_groups: Optional[GetCallGroups] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[CallGroupsResult]:
        """Get grouped call metrics

         For more information on Application Analyze please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-analyze.

        :param fill_time_series: If enabled, fill the missing data points in the metric result with timestamp and value 0.
        :type fill_time_series: bool
        :param get_call_groups:
        :type get_call_groups: GetCallGroups
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_call_group_serialize(
            fill_time_series=fill_time_series,
            get_call_groups=get_call_groups,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CallGroupsResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_call_group_without_preload_content(
        self,
        fill_time_series: Annotated[Optional[StrictBool], Field(description="If enabled, fill the missing data points in the metric result with timestamp and value 0.")] = None,
        get_call_groups: Optional[GetCallGroups] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get grouped call metrics

         For more information on Application Analyze please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-analyze.

        :param fill_time_series: If enabled, fill the missing data points in the metric result with timestamp and value 0.
        :type fill_time_series: bool
        :param get_call_groups:
        :type get_call_groups: GetCallGroups
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_call_group_serialize(
            fill_time_series=fill_time_series,
            get_call_groups=get_call_groups,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CallGroupsResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_call_group_serialize(
        self,
        fill_time_series,
        get_call_groups,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if fill_time_series is not None:
            
            _query_params.append(('fillTimeSeries', fill_time_series))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if get_call_groups is not None:
            _body_params = get_call_groups


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/application-monitoring/analyze/call-groups',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_correlated_traces(
        self,
        correlation_id: Annotated[str, Field(min_length=0, strict=True, max_length=128, description="Here, the `backendTraceId` is typically used which can be obtained from the `Get all beacons` API endpoint for website and mobile app monitoring. For XHR, fetch, or HTTP beacons, the `beaconId` retrieved from the same API endpoint can also serve as the `correlationId`. ")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[BackendTraceReference]:
        """Resolve Trace IDs from Monitoring Beacons.

        Resolves backend trace IDs using correlation IDs from website and mobile app monitoring beacons. For more information on Application Analyze please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-analyze.

        :param correlation_id: Here, the `backendTraceId` is typically used which can be obtained from the `Get all beacons` API endpoint for website and mobile app monitoring. For XHR, fetch, or HTTP beacons, the `beaconId` retrieved from the same API endpoint can also serve as the `correlationId`.  (required)
        :type correlation_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_correlated_traces_serialize(
            correlation_id=correlation_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[BackendTraceReference]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_correlated_traces_with_http_info(
        self,
        correlation_id: Annotated[str, Field(min_length=0, strict=True, max_length=128, description="Here, the `backendTraceId` is typically used which can be obtained from the `Get all beacons` API endpoint for website and mobile app monitoring. For XHR, fetch, or HTTP beacons, the `beaconId` retrieved from the same API endpoint can also serve as the `correlationId`. ")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[BackendTraceReference]]:
        """Resolve Trace IDs from Monitoring Beacons.

        Resolves backend trace IDs using correlation IDs from website and mobile app monitoring beacons. For more information on Application Analyze please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-analyze.

        :param correlation_id: Here, the `backendTraceId` is typically used which can be obtained from the `Get all beacons` API endpoint for website and mobile app monitoring. For XHR, fetch, or HTTP beacons, the `beaconId` retrieved from the same API endpoint can also serve as the `correlationId`.  (required)
        :type correlation_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_correlated_traces_serialize(
            correlation_id=correlation_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[BackendTraceReference]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_correlated_traces_without_preload_content(
        self,
        correlation_id: Annotated[str, Field(min_length=0, strict=True, max_length=128, description="Here, the `backendTraceId` is typically used which can be obtained from the `Get all beacons` API endpoint for website and mobile app monitoring. For XHR, fetch, or HTTP beacons, the `beaconId` retrieved from the same API endpoint can also serve as the `correlationId`. ")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Resolve Trace IDs from Monitoring Beacons.

        Resolves backend trace IDs using correlation IDs from website and mobile app monitoring beacons. For more information on Application Analyze please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-analyze.

        :param correlation_id: Here, the `backendTraceId` is typically used which can be obtained from the `Get all beacons` API endpoint for website and mobile app monitoring. For XHR, fetch, or HTTP beacons, the `beaconId` retrieved from the same API endpoint can also serve as the `correlationId`.  (required)
        :type correlation_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_correlated_traces_serialize(
            correlation_id=correlation_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[BackendTraceReference]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_correlated_traces_serialize(
        self,
        correlation_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if correlation_id is not None:
            
            _query_params.append(('correlationId', correlation_id))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/application-monitoring/analyze/backend-correlation',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_trace_download(
        self,
        id: StrictStr,
        retrieval_size: Optional[Annotated[int, Field(le=10000, strict=True, ge=1)]] = None,
        offset: Optional[StrictInt] = None,
        ingestion_time: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> TraceDownloadResult:
        """Get trace detail

        Use this API endpoint if one wants to retrive comprehensive details of a particular trace. For more information on Application Analyze please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-analyze.

        :param id: (required)
        :type id: str
        :param retrieval_size:
        :type retrieval_size: int
        :param offset:
        :type offset: int
        :param ingestion_time:
        :type ingestion_time: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_trace_download_serialize(
            id=id,
            retrieval_size=retrieval_size,
            offset=offset,
            ingestion_time=ingestion_time,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TraceDownloadResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_trace_download_with_http_info(
        self,
        id: StrictStr,
        retrieval_size: Optional[Annotated[int, Field(le=10000, strict=True, ge=1)]] = None,
        offset: Optional[StrictInt] = None,
        ingestion_time: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[TraceDownloadResult]:
        """Get trace detail

        Use this API endpoint if one wants to retrive comprehensive details of a particular trace. For more information on Application Analyze please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-analyze.

        :param id: (required)
        :type id: str
        :param retrieval_size:
        :type retrieval_size: int
        :param offset:
        :type offset: int
        :param ingestion_time:
        :type ingestion_time: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_trace_download_serialize(
            id=id,
            retrieval_size=retrieval_size,
            offset=offset,
            ingestion_time=ingestion_time,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TraceDownloadResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_trace_download_without_preload_content(
        self,
        id: StrictStr,
        retrieval_size: Optional[Annotated[int, Field(le=10000, strict=True, ge=1)]] = None,
        offset: Optional[StrictInt] = None,
        ingestion_time: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get trace detail

        Use this API endpoint if one wants to retrive comprehensive details of a particular trace. For more information on Application Analyze please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-analyze.

        :param id: (required)
        :type id: str
        :param retrieval_size:
        :type retrieval_size: int
        :param offset:
        :type offset: int
        :param ingestion_time:
        :type ingestion_time: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_trace_download_serialize(
            id=id,
            retrieval_size=retrieval_size,
            offset=offset,
            ingestion_time=ingestion_time,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TraceDownloadResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_trace_download_serialize(
        self,
        id,
        retrieval_size,
        offset,
        ingestion_time,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        if retrieval_size is not None:
            
            _query_params.append(('retrievalSize', retrieval_size))
            
        if offset is not None:
            
            _query_params.append(('offset', offset))
            
        if ingestion_time is not None:
            
            _query_params.append(('ingestionTime', ingestion_time))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/application-monitoring/v2/analyze/traces/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_trace_groups(
        self,
        fill_time_series: Annotated[Optional[StrictBool], Field(description="If enabled, fill the missing data points in the metric result with timestamp and value 0.")] = None,
        get_trace_groups: Optional[GetTraceGroups] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> TraceGroupsResult:
        """Get grouped trace metrics

        In this endpoint, the supported `groupbyTag` are `trace.endpoint.name` and `trace.service.name`.  For more information on Application Analyze please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-analyze.

        :param fill_time_series: If enabled, fill the missing data points in the metric result with timestamp and value 0.
        :type fill_time_series: bool
        :param get_trace_groups:
        :type get_trace_groups: GetTraceGroups
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_trace_groups_serialize(
            fill_time_series=fill_time_series,
            get_trace_groups=get_trace_groups,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TraceGroupsResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_trace_groups_with_http_info(
        self,
        fill_time_series: Annotated[Optional[StrictBool], Field(description="If enabled, fill the missing data points in the metric result with timestamp and value 0.")] = None,
        get_trace_groups: Optional[GetTraceGroups] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[TraceGroupsResult]:
        """Get grouped trace metrics

        In this endpoint, the supported `groupbyTag` are `trace.endpoint.name` and `trace.service.name`.  For more information on Application Analyze please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-analyze.

        :param fill_time_series: If enabled, fill the missing data points in the metric result with timestamp and value 0.
        :type fill_time_series: bool
        :param get_trace_groups:
        :type get_trace_groups: GetTraceGroups
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_trace_groups_serialize(
            fill_time_series=fill_time_series,
            get_trace_groups=get_trace_groups,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TraceGroupsResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_trace_groups_without_preload_content(
        self,
        fill_time_series: Annotated[Optional[StrictBool], Field(description="If enabled, fill the missing data points in the metric result with timestamp and value 0.")] = None,
        get_trace_groups: Optional[GetTraceGroups] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get grouped trace metrics

        In this endpoint, the supported `groupbyTag` are `trace.endpoint.name` and `trace.service.name`.  For more information on Application Analyze please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-analyze.

        :param fill_time_series: If enabled, fill the missing data points in the metric result with timestamp and value 0.
        :type fill_time_series: bool
        :param get_trace_groups:
        :type get_trace_groups: GetTraceGroups
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_trace_groups_serialize(
            fill_time_series=fill_time_series,
            get_trace_groups=get_trace_groups,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TraceGroupsResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_trace_groups_serialize(
        self,
        fill_time_series,
        get_trace_groups,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if fill_time_series is not None:
            
            _query_params.append(('fillTimeSeries', fill_time_series))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if get_trace_groups is not None:
            _body_params = get_trace_groups


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/application-monitoring/analyze/trace-groups',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_traces(
        self,
        get_traces: Optional[GetTraces] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> TraceResult:
        """Get all traces

         For more information on Application Analyze please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-analyze.

        :param get_traces:
        :type get_traces: GetTraces
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_traces_serialize(
            get_traces=get_traces,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TraceResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_traces_with_http_info(
        self,
        get_traces: Optional[GetTraces] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[TraceResult]:
        """Get all traces

         For more information on Application Analyze please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-analyze.

        :param get_traces:
        :type get_traces: GetTraces
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_traces_serialize(
            get_traces=get_traces,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TraceResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_traces_without_preload_content(
        self,
        get_traces: Optional[GetTraces] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get all traces

         For more information on Application Analyze please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-analyze.

        :param get_traces:
        :type get_traces: GetTraces
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_traces_serialize(
            get_traces=get_traces,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TraceResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_traces_serialize(
        self,
        get_traces,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if get_traces is not None:
            _body_params = get_traces


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/application-monitoring/analyze/traces',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )


